//  Copyright (C) 2014 - Davide Cappucci
//
//  This file is part of the Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.
//

m = mode();
mode(7)
//
// Forward kinematics is the problem of solving the Cartesian position and 
// orientation of a mechanism given knowledge of the kinematic structure and 
// the joint coordinates.
//
// We will work with a model of the Puma 560 robot
mdl_puma560

// Consider the Puma 560 example again, and the joint coordinates of zero,
// which are defined by the script

qz

// The forward kinematics may be computed using fkine() method of the
// p560 robot object

fkine(p560,qz)

// returns the homogeneous transform corresponding to the last link of the 
// manipulator
//
// fkine() can also be used with a time sequence of joint coordinates, or 
// trajectory, which is generated by jtraj()

t = [0:.056:2];    // generate a time vector
q = jtraj(qz, qready, t); // compute the joint coordinate trajectory

mprintf("\n q [%s] : %dx%d",typeof(q),size(q,1),size(q,2));

// then the homogeneous transform for each set of joint coordinates is given by

T = fkine(p560,q);

mprintf("\n T [%s] : %dx%dx%d",typeof(T),size(T,1),size(T,2),size(T,3))

// where T is a 3-dimensional matrix, the first two dimensions are a 4x4 
// homogeneous transformation and the third dimension is time.
//
// For example, the first point is

T(:,:,1)

// and the tenth point is

T(:,:,10)
//
// Elements (1:3,4) correspond to the X, Y and Z coordinates respectively, and 
// may be plotted against time
scf(100001); subplot(3,1,1); plot(t, squeeze(T(1,4,:)));
xlabel('Time (s)');
ylabel('X (m)')
subplot(3,1,2); plot(t, squeeze(T(2,4,:)));
xlabel('Time (s)');
ylabel('Y (m)')
subplot(3,1,3); plot(t, squeeze(T(3,4,:)));
xlabel('Time (s)');
ylabel('Z (m)')

// or we could plot X against Z to get some idea of the Cartesian path followed
// by the manipulator.

scf(100002); plot(squeeze(T(1,4,:)), squeeze(T(3,4,:)));
xlabel('X (m)')
ylabel('Z (m)')
xgrid();
mode(m);
return;
