//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function v = %Link_e(field, l)
    v = [];
    if typeof(l)=="Link" then
        select convstr(field,"l")
        case "m" then
            v=l.m
        case "r" then
            v=l.r
        case "I" then
            v=l.I
        case "Jm" then
            v=Jm
        case "G" then
            v=l.G
        case "B" then
            v=l.B
        case "Tc" then
            v=l.Tc
        case "qlim" then
            v=l.qlim
            //-------------------------------------------------------//
            //  Methods
            //-------------------------------------------------------//
        case "rp" then
            v = RP(l);
        case "isrevolute" then 
            v = isrevolute(l);
        case "isprismatic" then 
            v = isprismatic(l);
        case "display"
            v = l
        case "char" then
            print_link(l);
        //case "dyn" then
        //    dyn(l);
        //
        // legacy DH/DYN support
        //
        case "dyn" then                  // additional method, not a field
            if isempty(l.m) then
                error("cannot return the legacy DYN matrix: non-optional link mass parameter missing");
            elseif isempty(l.r) then
                error("cannot return the legacy DYN matrix: non-optional link COG parameter missing");
            elseif isempty(l.I) then
                error("cannot return the legacy DYN matrix: non-optional link inertia parameter missing");
            elseif isempty(l.Jm) then
                error("cannot return the legacy DYN matrix: non-optional motor inertia parameter missing");
            else
                G = l.G; if isempty(G) then G = 1; end
                v = [l.alpha, l.a, l.theta, l.d, l.sigma,..
                    l.m, l.r, l.I(1,1), l.I(2,2), l.I(3,3), l.I(1,2), l.I(2,3), l.I(1,3), l.Jm, G, l.B, l.Tc];
            end
        case "dh" then                   // additional method, not a field
            v = [l.alpha, l.a, l.theta, l.d, l.sigma];
        case "friction" then
            error("Use ''friction(link,qd)''");  
        else
            error('Invalid field');
        end
    end
endfunction
