//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function qp = %Quat_m_s(q1, q2)
    // Multiply a quaternion object
    // 
    // Description
    // <literal>Q*V</literal>     is a vector formed by rotating the vector V by the quaternion Q.
    // <literal>Q*S</literal>     is the element-wise multiplication of quaternion elements by the scalar S.
    //
    // Notes:
    // - Overloaded operator '*'
    //
    // See also 
    // Quaternion
    // %Quat_m_Quat
    // %s_m_Quat 
    // qinv
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    
    if length(q2) == 3
        qp = q1 * Quaternion([0 q2(:)']) * qinv(q1);
        qp = qp.v(:);
    elseif length(q2) == 1
        qp = Quaternion( qdouble(q1)*q2);
    else
        error('quaternion-vector product: must be a 3-vector or scalar');
    end

endfunction
