//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [h] = create_new_robot(robot,opt)
	
    cfh = gcf();

    h = [];
    h.mag = opt.mag;
    //    h.opt = opt;    
    
    set(cfh,"pixel_drawing_mode","andInverted")
//    cfh.background= color("grey70")
    cfh.background= color("white")

    currentAxes = cfh.children;

    if opt.hold == %t then
        // if current figure has hold on
        // calculate the new size of the workspace
        old_bounds = cfh.children.data_bounds;
        if numcols(old_bounds)==2 & numrows(old_bounds)==2 then
            old_bounds = matrix(old_bounds,1,4)
            for i=1:4
                if modulo(i,2)== 0 then
                    bounds(i) = max(opt.workspace(i),old_bounds(i));
                else
                     bounds(i) = min(opt.workspace(i),old_bounds(i));
               end
            end
            currentAxes.data_bounds = bounds';
        elseif numcols(old_bounds)==3 & numrows(old_bounds)==2
            old_bounds = matrix(old_bounds,1,6);
            for i=1:6
                if modulo(i,2)== 0 then
                    bounds(i) = max(opt.workspace(i),old_bounds(i));
                else
                     bounds(i) = min(opt.workspace(i),old_bounds(i));
               end
            end
            currentAxes.data_bounds = bounds';
        else
            error("create_new_robot: wrong axes data bounds");
        end
    else
        currentAxes.data_bounds = opt.workspace;
    end
    
    currentAxes.axes_visible="on";
    currentAxes.x_label.text="X";
    currentAxes.y_label.text="Y";
    currentAxes.z_label.text="Z";
    currentAxes.grid=[3,3,3]; 
    currentAxes.view="3d";
    currentAxes.foreground = -1
    currentAxes.background = -1

    zlim = currentAxes.data_bounds;
    h.zmin = zlim(1);

    if opt.base
        b = transl(robot.base);
        xsegs([b(1);b(1)], [b(2);b(2)], [h.zmin;b(3)]);
        colorLineId = addcolor(name2rgb('red')/255);
 
            currentAxes.children(1).segs_color=colorLineId;
            currentAxes.children(1).thickness=robot.lineopt(2);
        
    end


    if opt.name
        b = transl(robot.base);
        xstring(b(1),b(2)-opt.mag,[' ' robot.name])
        t=get("hdl")   //get the handle of the newly created object
        //fontid = 10: SansSerif, Bold, Italic //fontsize =1
        t.font_size=1;
        t.font_style=10;
    end

    // set the link parameters
    param3d([0;0],[0;0],[0;0]);
    segHandler =get("hdl");
    segHandler.foreground=15;
    segHandler.thickness=robot.lineopt(2);
    h.links = segHandler
    h.links.user_data = list();
    
    if opt.shadow then
        param3d([0;0],[0;0],[0;0]);
        colorShadowId = addcolor(name2rgb(robot.shadowopt(1))/255);
        segHandler =get("hdl");
        segHandler.foreground=colorShadowId;
        segHandler.thickness=robot.shadowopt(2);
        h.shadow =segHandler;
        h.shadow.user_data=list();
    end

    if opt.wrist then
        xsegs([0;0],[0;0],[0;0]);
        colorShadowId = addcolor(name2rgb('red')/255);
        segHandler =get("hdl");
        segHandler.segs_color=colorShadowId;
        h.x =segHandler;
        xsegs([0;0],[0;0],[0;0]);
        colorShadowId = addcolor(name2rgb('green')/255);
        segHandler =get("hdl");
        segHandler.segs_color=colorShadowId;
        h.y =segHandler;
        xsegs([0;0],[0;0],[0;0]);
        colorShadowId = addcolor(name2rgb('blue')/255);
        segHandler =get("hdl");
        segHandler.segs_color=colorShadowId;
        segHandler.thickness=1;
        h.z =segHandler;

        //fontid = 10: SansSerif, Bold, Italic //fontsize =1
        xstring(0,0,['x'])
        t=get("hdl")   //get the handle of the newly created object
        t.font_size=1;
        t.font_style=10;
        h.xt= t

        xstring(0, 0, ['y']);
        t=get("hdl")   //get the handle of the newly created object
        t.font_size=1;
        t.font_style=10;
        h.yt = t
        // opt.wristlabel(2));//) 'FontWeight', 'bold', 'HorizontalAlignment', 'Center');

        xstring(0, 0, ['z']);
        t=get("hdl")   //get the handle of the newly created object
        t.font_size=1;
        t.font_style=10;
        h.zt = t;
    end

    //
    // display cylinders (revolute) or boxes (pristmatic) at
    // each joint, as well as axis centerline.
    //
    L = robot.links;
    //cyColor = color(opt.cylinder(1),opt.cylinder(2),opt.cylinder(3));
    h.joint = [];
    h.jointaxis = [];
    h.jointlabel = [];
    h.q = [];
    h.casing =[];

    for i=1:robot.n

        if  opt.joints then
            //cylinder or box to represent the joint
            if L(i).sigma == 0 then
                N = 16;
            else
                N = 4;
            end

            // define the vertices of the cylinder
            [xc,yc,zc] = rt_cylinder(opt.mag/4, N);
            //[xc,yc,zc] = cylinder(opt.mag/4, N);
            zc(zc==0) = -opt.mag/2;
            zc(zc==1) = opt.mag/2;

            // add the surface object
            surf(xc,yc,zc);
            h.joint = [h.joint; gce()]; 
            h.joint(i).thickness = 1;
            //joint.foreground = color("black");
            colorJointId = addcolor(opt.cylinder);
            
            h.joint(i).foreground = 8;
            h.joint(i).color_mode = 32; //colorJointId
            h.joint(i).color_flag = 2; //flat shading


            // build a matrix of coordinates so we
            // can transform the cylinder in rt_animate()
            // and hang it off the cylinder
            datax = getfield(2, h.joint(i).data);
            datay = getfield(3, h.joint(i).data);
            dataz = getfield(4, h.joint(i).data);
            h.joint(i).user_data = [datax(:)'; datay(:)'; dataz(:)'; ones(1,4*N)];
             
            if opt.jaxes
                // add a dashed line along the axis
                param3d([0;0],[0;0],[0;0]);
                segHandler =get("hdl");
                segHandler.foreground=32;
                segHandler.line_style = 2

                h.jointaxis = [h.jointaxis; segHandler];
                h.jointaxis(i).line_style = 4;


                xstring(0, 0, [sci2exp(i)]);
                h.jointlabel = [h.jointlabel;gce()];
                h.jointlabel(i).alignment = 'center';
            end

                xstring(0, 0, [sci2exp(i)]);
                h.jointlabel = [h.jointlabel;gce()];
                h.jointlabel(i).alignment = 'center';
            
        end
    end

    currentAxes.box="back_half";
endfunction
