//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function dyn(l)
// Show inertial properties of link
// 
// Description
// <literal>L.dyn()</literal> displays the inertial properties of the link object in a multi-line
// format. The properties shown are mass, centre of mass, inertia, friction,
// gear ratio and motor properties.
// 
// If L is a vector of Link or a SerialLink object show properties for each link.
// 
// Bibliography
// [1] Robotics, Vision & Control, Chap 7, P. Corke, Springer 2011.   
//
// See also
//  Link
//	SerialLink 
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci


    [%nargout,%nargin] =argn(0);
    if typeof(l) == "Link" then
        disp(print_link(l));
        //m,r,I,Jm,G,B,Tc
        if ~isempty(l(9))
            mprintf('  m    = %f\n', l(9));
        end
        if ~isempty(l(10))

            mprintf('  r    = %f %f %f\n', l(10));
        end
        if ~isempty(l(11))
            mprintf('  I    = | %f %f %f |\n', l(11)(1,:));
            mprintf('         | %f %f %f |\n', l(11)(2,:));
            mprintf('         | %f %f %f |\n', l(11)(3,:));
        end
        if ~isempty(l(12))
            mprintf('  Jm   = %f\n', l(12));
        end
        if ~isempty(l(14))
            mprintf('  Bm   = %f\n', l(14));
        end
        if ~isempty(l(15))
            mprintf('  Tc   = %f(+) %f(-)\n', l(15)(1), l(15)(2));
        end
        if ~isempty(l(13))
            mprintf('  G    = %f\n', l(13));
        end
        if ~isempty(l(16))
             mprintf('  qlim = %f to %f\n', l(16)(1), l(16)(2));
        end

    elseif typeof(l)=="Robot" then
        for i=1:l.n,
            mprintf("\n|=================Link %d==================================================|\n",i)
            dyn(l.links(i));
        end
    else
    end

endfunction

