//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function h = isvec(v, l)
    // Test if argument is a vector
    //
    // Calling Sequence
    // isvec(v)
    // isvec(v,l)
    //
    // Description
    // ISVEC(V) is true (1) if the argument V is a 3-vector, else false (0).
    //
    // ISVEC(V, L) is true (1) if the argument V is a vector of length L,
    // either a row- or column-vector.  Otherwise false (0).
    //
    // <refsection>
    // <title>Notes</title>
    // <itemizedlist>
    // <listitem><para>differs from Scilab builtin function ISVECTOR, the latter 
    // returns true for the case of a scalar, ISVEC does not.</para></listitem>
    // </itemizedlist>
    // </refsection>
    //
    // See also 
    // ishomog
    // isrot
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    if argn(2) == 1
        l = 3;
    end
    d = size(v);
    h =  length(d) == 2 & min(d) == 1 & length(v) == l ;

endfunction
