//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function [x, y, typ] = jacobn_if(job, arg1, arg2)

    x = [];
    y = [];
    typ = [];
    
    in_label = ['q']; 
	out_label = ['J'];
	style = 'blockWithLabel;fillColor=red;displayedLabel=N<BR><big> J;fontStyle=1;fontSize=12';
    select job,
        case "set" then
            x = arg1;
            graphics = arg1.graphics;
            exprs = graphics.exprs;
            model = arg1.model;
            while %T do,
                [ok, robot_name, exprs] = scicos_getvalue(..
                    "Robot jacobian for end-effector coordinates",..
                    ["Robot object"],..
                    list("str", -1),..
                    exprs)
				robot = evstr(robot_name)
                if ok & typeof(robot) ~= "Robot" then
                    // not a robot
                    messagebox("invalid block parameter: required a robot object");
                    ok = %F;
                elseif ok & robot.n == 0 then
                    // a robot model to be simulated still must be specified
                    ok = %F;
                end
                if ~ok then
                    break;
                end
                [model, graphics, ok] = set_io(model, graphics, list([robot.n, 1], 1), list([6, robot.n], 1), [], []);
                if ok then
                    model.opar = list(robot.links, robot.base, robot.tool, robot.offset, robot.mdh);
                    graphics.exprs = exprs;
                    x.graphics = graphics;
                    x.model = model;
                    break
                end
            end

        case "define" then
            model = scicos_model();
            model.sim = list("jacobn_cf", 5);
            model.in = -1;
            model.in2 = 1;
            model.intyp = 1;
            model.out = 6;
            model.out2 = -1;
            model.outtyp = 1;
            model.opar = list();
            model.blocktype = "d";
            model.dep_ut = [%T %F];
            exprs = "SerialLink()";
            x = standard_define([4 4], model, exprs, "");
            x.graphics.style = style;
			x.graphics.out_label = out_label;
			x.graphics.in_label = in_label;

    end

endfunction
