//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function mdl_phantomx()
// Create model of PhantomX pincher manipulator
//
// Calling Sequence
// mdl_phantomx
//
// Description
// Script creates the workspace variable px which describes the 
// kinematic characteristics of a PhantomX Pincher Robot, a 4 joint hobby
// class  manipulator by Trossen Robotics.
//
// Also define the workspace vectors:
//
//   qz         zero joint angle configuration
//
// <refsection>
// <title>Notes</title>
// <itemizedlist>
// <listitem><para>the x-axis is forward, and the z-axis is upwards.</para></listitem>
// <listitem><para>uses standard DH conventions.</para></listitem>
// <listitem><para>Tool centrepoint is middle of the fingertips.</para></listitem>
// <listitem><para>all translational units in mm.</para></listitem>
// </itemizedlist>
// </refsection>
//
// Bibliography
// [1] http://www.trossenrobotics.com/productdocs/assemblyguides/phantomx-basic-robot-arm.html
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci

L = list();
L(1) = Link('d', 40, 'alpha', -%pi/2);
L(2) = Link('a', -105, 'alpha', %pi, 'offset', %pi/2);
L(3) = Link('a', -105);
L(4) = Link('a', -105);

// Note alpha_2 = pi, needed to account for rotation axes of joints 3 and 4 having
// opposite sign to joint 2.
//
// s='Rz(q1) Tz(L1) Ry(q2) Tz(L2) Ry(q3) Tz(L3) Ry(q4) Tz(L4)'
// DHFactor(s)
//
px = SerialLink(L, 'name', 'PhantomX', 'manuf', 'Trossen Robotics');
qz = [0 0 0 0];
px.tool = trotz(-%pi/2) * trotx(%pi/2);

    [px,qz]=resume(px,qz)
endfunction
