//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function  r2 = perturb(r, p)
    // Perturb robot parameters
    //
    // Calling Sequence
    // RP = ROBOT.perturb()
    // RP = perturb(ROBOT,P)
    //
    // Description 
    // RP = perturb(ROBOT,P) is a new robot object in which the dynamic parameters (link
    // mass and inertia) have been perturbed.  The perturbation is multiplicative so 
    // that values are multiplied by random numbers in the interval (1-P) to (1+P).
    // The name string of the perturbed robot is prefixed by 'P/'.
    //
    // Useful for investigating the robustness of various model-based control 
    // schemes. For example to vary parameters in the range +/- 10 percent is:
    //    r2 = perturb(p560,0.1);
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci

    [%nargout,%nargin]=argn(0);

    if %nargin == 1
        p = 0.1;	// 10 percent disturb by default
    end

    r2 = SerialLink(r);

    links = r2.links;
    for i=1:r.n
        s = (2*rand()-1)*p + 1;
        links(i).m = links(i).m * s;

        s = (2*rand()-1)*p + 1;
        links(i).I = links(i).I * s;
    end

    r2.name = ['P/'+r.name];
endfunction
