//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function n = qnorm(q)
    // Quaternion magnitude
    //
    // Description
    // <literal>QN = Q.norm()</literal> is the equivalent of 
    // <literal>QN = qnorm(Q)</literal>. 
    // It is the scalar norm or magnitude 
    // of the quaternion Q.  
    //
    // <refsection>
	// <title>Notes</title>
	// <itemizedlist>
    // <listitem><para>This is the Euclidean norm of the quaternion written 
    // as a 4-vector.</para></listitem>
    // <listitem><para>A unit-quaternion has a norm of one.</para></listitem>
    // </itemizedlist>
	// </refsection>
    //
    // See also 
    // Quaternion
    // qunit
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    if typeof(q) == "constant" then
        n = norm(q);
    elseif typeof(q) == "Quat"
        n = norm(qdouble(q));
    end
    
endfunction
