//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function hg = qplot(Q, varargin)
    // Plot a quaternion object 
    //
    // Description
    // <literal>qplot(Q,options)</literal> plots the quaternion as a rotated 
    // coordinate frame.
    // Options are passed to trplot and include:
    //
    // Parameters
    //
    // 'color',C      :  The color to draw the axes, MATLAB colorspec C
    // 'frame',F      :  The frame is named {F} and the subscript on the axis labels is F.
    // 'view',V       :  Set plot view parameters V=[az el] angles, or 'auto' 
    //                    for view toward origin of coordinate frame
    //
    // Authors
    // Copyright (C) 1999-2002, by Peter I. Corke
    // Copyright (C) 2007  Interdepartmental Research Center "E. Piaggio", University of Pisa
    // Copyright (C) 2014-20 - Davide Cappucci

    // See also 
    // trplot

    [%nargout,%nargin] = argn(0);

    drawlater();

    // default options for axes plotting
    plotoptions.x.color = "black";
    plotoptions.y.color = "black";
    plotoptions.z.color = "black";

    if %nargin > 1 then

        i = 1;
        if (~isempty(varargin) & typeof(varargin(1))=="string") then
            while i <= length(varargin),
                select convstr(varargin(i),"l")
                case "x" then
                    plotoptions.x.color = varargin(i+1);
                    i = i+1;
                case "y" then
                    plotoptions.y.color = varargin(i+1);
                    i = i+1;
                case "z" then
                    plotoptions.z.color = varargin(i+1);
                    i = i+1;
                else
                    error("unknown option: %s\n",varargin(i));
                end
                i = i+1;
            end
        end

    end

    currax = gca();
    viewpoint = currax.rotation_angles;
    old_auto_clear = currax.auto_clear;
    currax.view = "3d"
    currax.data_bounds = [-1 -1 -1;1 1 1];
    currax.box = "on"

    currax.auto_clear = "off";
    if (length(Q)> 1) & typeof(Q)=="list" then
        o = [0 0 0]';
        x1 = Q(1)*[1 0 0]';
        y1 = Q(1)*[0 1 0]';
        z1 = Q(1)*[0 0 1]';

        xsegs([0;x1(1)], [0;x1(2)], [0;x1(3)])// plotoptions.x.color, 1, "copy");
        a = gce();
        a.segs_color =  addcolor(name2rgb(plotoptions.x.color)/255);

        //    xtitle(currax.parent, x1(1), x1(2), "X")//  "copy", x1(3));
        xsegs( [0;y1(1)], [0;y1(2)], [0;y1(3)])//  plotoptions.y.color, 1, "copy");
        //    xtitle(currax.parent, y1(1), y1(2), "Y")//  "copy", y1(3));
        xsegs([0;z1(1)], [0;z1(2)], [0;z1(3)])//  plotoptions.z.color, 1, "copy");
        sleep(3000)
        for i=2:length(Q),
            disp("ciao")
            x1 = Q(i)*[1 0 0]';
            y1 = Q(i)*[0 1 0]';
            z1 = Q(i)*[0 0 1]';
            
            a=gca();
            //            a.children(1).data=[0,0,0;x1(1),x1(2),x1(3)];
            //            a.children(2).data=[0,0,0;y1(1),y1(2),y1(3)];
            //            a.children(3).data=[0,0,0;z1(1),z1(2),z1(3)];
//            move(a.children(1),[x1(1),y1(1),z1(1)]);
//            move(a.children(2),[x1(2),y1(2),z1(2)]);
//            move(a.children(3),[x1(3),y1(3),z1(3)]);
////            move(a.children(1),[x1(1),x1(2),x1(3)]);
////            disp(a.children(1))
////            move(a.children(2),[y1(1),y1(2),y1(3)]);
////            move(a.children(3),[z1(1),z1(2),z1(3)]);                        
////            
            move(a.children(1),[.1,0,0]);
            move(a.children(2),[.1,0,0]);
            move(a.children(3),[.1,0,0]);
            drawnow();
            sleep(3000)
        end
    else
        // create unit vectors
        o = [0 0 0]';
        x1 = Q*[1 0 0]';
        y1 = Q*[0 1 0]';
        z1 = Q*[0 0 1]';
        xsegs([0;x1(1)], [0;x1(2)], [0;x1(3)])// plotoptions.x.color, 1, "copy");
        a = gce();
        a.segs_color =  addcolor(name2rgb(plotoptions.x.color)/255);

        //    xtitle(currax.parent, x1(1), x1(2), "X")//  "copy", x1(3));
        xsegs( [0;y1(1)], [0;y1(2)], [0;y1(3)])//  plotoptions.y.color, 1, "copy");
        //    xtitle(currax.parent, y1(1), y1(2), "Y")//  "copy", y1(3));
        xsegs([0;z1(1)], [0;z1(2)], [0;z1(3)])//  plotoptions.z.color, 1, "copy");
        //    xtitle(currax.parent, z1(1), z1(2), "Z")//  "copy", z1(3));
        xgrid();
    end
    // maintain point of view
    currax.rotation_angles = viewpoint;
    currax.view = "3d"

    currax.x_label.text = "X";
    currax.y_label.text = "Y";
    currax.z_label.text = "Z";

    // maintain auto_clear
    currax.auto_clear = old_auto_clear;

    drawnow();
    hg = currax;

endfunction

