//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function R = roty(t, deg)
    // Rotation about Y axis
    //
    // Calling Sequence
    //  R = roty(THETA)
    //  R = roty(THETA, 'deg')
    //
    // Description
    // <literal>R = ROTY(THETA)</literal> is a rotation matrix representing a rotation of THETA 
    // radians about the x-axis.
    //
    // <literal>R = ROTY(THETA, 'deg')</literal> as above but THETA is in degrees.
    //
    // See also 
    // rotx
    // rotz
    //
    // Authors
    // Copyright (C) 1993-2011 - Peter I. Corke
    // Copyright (C) 2014-20 - Davide Cappucci
    
    [%nargout,%nargin] = argn(0);

    if (%nargin > 1 & deg == 'deg') then
        t = t *%pi/180;
    end

    ct = cos(t);
    st = sin(t);
    R = [
        ct  0   st
        0   1   0
       -st  0   ct
       ];
endfunction
