//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function l=set_Tc(l, v)
// Set Coulomb friction
//
// Description
// <literal>L.Tc = F</literal> set Coulomb friction parameters to [F -F], for a symmetric
// Coulomb friction model.
//
// <literal>L.Tc = [FP FM]</literal> set Coulomb friction to [FP FM], for an asymmetric
// Coulomb friction model. FP>0 and FM<0.
//
// Bibliography
// [1] Robotics, Vision & Control, Chap 7, P. Corke, Springer 2011. 
//
// See also
//	Link
//  friction.
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci

    if isempty(v)
        return;
    end
    
    if length(v) == 1
        l.Tc = [v -v]; 
    elseif length(v) == 2
        if v(1) < v(2)
            error('Link: badarg: Coulomb friction is [Tc+ Tc-]');
        end
        l(15) = v;     
    else
        error('Coulomb friction vector can have 1 (symmetric) or 2 (asymmetric) elements only')
    end
endfunction
