//  Copyright (C) 2014-20 - Davide Cappucci
//
//  This file is part of Robotics Toolbox for Scilab.
//
//  Robotics Toolbox is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Robotics Toolbox is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
//  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
//  by Peter Corke.

function J = zyz2jac(phi, theta, psi)
// Euler angle rate Jacobian
//
// Calling Sequence
// J = ZYZ2JAC(EUL)
// J = ZYZ2JAC(PHI, THETA, PSI)
//
// Description
// J = ZYZ2JAC(EUL) is a Jacobian matrix (3x3) that maps Euler angle rates to 
// angular velocity at the operating point EUL=[PHI, THETA, PSI]. 
//
// J = ZYZ2JAC(PHI, THETA, PSI) as above but the Euler angles are passed
// as separate arguments.
//
// <refsection>
// <title>Notes</title>
// <itemizedlist>
// <listitem><para>Used in the creation of an analytical Jacobian.</para></listitem>
// </itemizedlist>
// </refsection>
//
// See also
// rpy2jac
// jacobn
//
// Authors
// Copyright (C) 1993-2011 - Peter I. Corke
// Copyright (C) 2014-20 - Davide Cappucci

    [%nargout,%nargin] = argn(0);

    if length(phi) == 3
        // eul2jac([phi theta psi])
        theta = phi(2);
        psi = phi(3);
        phi = phi(1);
    elseif %nargin ~= 3
        error('zyz2jac:  bad arguments');
    end
    J = [ 0, -sin(phi), cos(phi)*sin(theta)
    0,  cos(phi), sin(phi)*sin(theta)
    1,        0,           cos(theta) ];  

endfunction
