/*  Copyright (C) 2014-20 - Davide Cappucci
*
*  This file is part of the Robotics Toolbox for Scilab.
*
*  Robotics Toolbox is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Robotics Toolbox is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
*  by Peter Corke.
*/

#include "api_scilab.h"
#include "Scierror.h"
#include "sciprint.h"
#include "robot.h"
#include "vmath.h"
#include <localization.h>

void print_vect(double *q,int length);
void print_matrix(double *matrix,int m,int n);
void rot_mat(Link *l,double th,double d, DHType type);
/* extern void newton_euler(Robot *robot, double *tau, double *qd, double *qdd, double *fext, int stride); */

//int frne(char *fname,unsigned long fname_len, void* pvApiCtx)
frne(char *fname, void* pvApiCtx)
{
	SciErr sciErr;
	int i       = 0;
	int iRet        = 0;
	int m1 = 0, n1 = 0;
	double *tau = NULL;
	int *piAddr     = NULL;
	Robot robot;
	Link    *l;
	double *Q = NULL;
	double *Qd = NULL;
	double *Qdd = NULL;
	double *Grav = NULL;
	double *Fext = NULL;
    int p,j,njoints = 0;
	int nq = 0;
	 
	int iType   = 0; /* GET ROBOT */
	
	/* GET LINKS */
	int iItem       = 0;
	int iRows           = 0;
	int iCols           = 0;
	double* pdblReal    = NULL;
	double* pdblImg    = NULL;
	int* piChild    = NULL;
	
	int ncols=0;
    int *piAddrQ = NULL;
    int *piAddrQd = NULL;
    int *piAddrQdd = NULL;
    int *piAddrGrav = NULL;
    int *piAddrFext = NULL;

	CheckInputArgument(pvApiCtx, 2, 6);

	sciErr = getVarAddressFromPosition(pvApiCtx, 1, &piAddr);
	if(sciErr.iErr)
	{
		printError(&sciErr, 0);
		return 0;
	}

	/*get_robot(1, NULL, piAddr, 0,robot);
	*	GET ROBOT
	*/


	sciErr = getVarType(pvApiCtx, piAddr, &iType);

	switch(iType)
	{
	case sci_tlist :
		/*
		 *	GET LINKS
		 */
			sciErr = getListItemNumber(pvApiCtx, piAddr, &iItem);
			if(sciErr.iErr)
			{
				printError(&sciErr, 0);
				return 0;
			}
			/* Robot (TList):
			 *	(1)
			 *	(2) name
			 *	(3) manuf
			 *	(4) comment
			 * 	(5) links
			 *	(6)	n
			 *	(7) mdh
			 *	(8) gravity
			 *	(9) base
			 *	(10) tool
			 *	(11) ....
			 */
			 
			for(i = 0 ; i < iItem ; i++)
			{
				sciErr = getListItemAddress(pvApiCtx, piAddr, i + 1, &piChild);
				if(sciErr.iErr)
				{
					printError(&sciErr, 0);
					return 0;
				}
				
				if (i==4){	//robot.links
					int iType   = 0;
					int j = 0;
					int k = 0;
					int l = 0;
					int dhSize = 0;
					int iLinksItem       = 0;
					int lengthOfLink = 0;
					int *piLinksAddr     = NULL;
					int* piLinksChild    = NULL;
					int *piLink = NULL;
					double *dh = malloc(26*sizeof(double));
					int lLink =0;
					int hLink =0;
					//iRet = get_info(_iRhs, _piAddr, piChild, i + 1);
					
					sciErr = getListItemAddress(pvApiCtx, piAddr, i + 1, &piLinksChild);
					if(sciErr.iErr)
					{
						printError(&sciErr, 0);
						return 0;
					}			
					sciErr = getListItemNumber(pvApiCtx, piLinksChild, &iLinksItem);
					if(sciErr.iErr)
					{
						printError(&sciErr, 0);
						return 0;
					}
					robot.links = (Link *) malloc(iLinksItem*sizeof(Link));
					for(j = 0 ; j < iLinksItem ; j++)
					{
						sciErr = getListItemAddress(pvApiCtx, piLinksChild, j + 1, &piLinksAddr);
						if(sciErr.iErr)
						{
							printError(&sciErr, 0);
							return 0;
						}			
						sciErr = getListItemNumber(pvApiCtx, piLinksAddr, &lengthOfLink);
						if(sciErr.iErr)
						{
							printError(&sciErr, 0);
							return 0;
						}
						sciErr = getVarType(pvApiCtx, piLinksAddr, &iType);
						if(sciErr.iErr){
							printError(&sciErr, 0);
							return 0;
						}
						
						for (k = 2; k <= lengthOfLink; ++k){
							sciErr = getMatrixOfDoubleInList(pvApiCtx, piLinksAddr, k, &iRows, &iCols, &pdblReal);
							if(sciErr.iErr)
							{
								printError(&sciErr, 0);
								return 0;
							}

							for (l=iRows*iCols-1; l>=0; --l){	
								dh[dhSize+l] = pdblReal[l];
					
							} 
							if (iRows*iCols>1)
								dhSize = dhSize+iRows*iCols;
							else
								dhSize = dhSize + 1;
						}

						robot.links[j] = create_link_2(dh);
						//print_Link(&robot.links[j]);
						dhSize = 0;
					} 
					free(dh);
				}

				if (i==5){	//robot.n
					//iRet = get_info(_iRhs, _piAddr, piChild, i + 1);
					sciErr = getMatrixOfDoubleInList(pvApiCtx,piAddr, i + 1, &iRows, &iCols, &pdblReal);
					if(sciErr.iErr)
					{
						printError(&sciErr, 0);
						return 0;
					}
					robot.njoints = (int)pdblReal[0];
				}
				if (i==6){
					sciErr = getMatrixOfDoubleInList(pvApiCtx, piAddr, i + 1, &iRows, &iCols, &pdblReal);
					if(sciErr.iErr)
					{
						printError(&sciErr, 0);
						return 0;
					}			
					robot.dhtype = (int)pdblReal[0];

				}
				if (i==7){
					sciErr = getMatrixOfDoubleInList(pvApiCtx, piAddr, i + 1, &iRows, &iCols, &pdblReal);
					if(sciErr.iErr)
					{
						printError(&sciErr, 0);
						return 0;
					}
					robot.gravity = (Vect *) malloc(sizeof(Vect));
					robot.gravity->x = pdblReal[0];
					robot.gravity->y = pdblReal[1];
					robot.gravity->z = pdblReal[2];
					
				}
			
			}
		/* FINE GET_LINK*/
		break;
	default :
		Scierror(999,"\tUnknown type for robot object\n");
		return 0;
	}
	/*FINE GET ROBOT*/
	njoints = robot.njoints;
#ifdef  DEBUG
	sciprint("\n|----------------Verifica Lettura Robot------------------|");
	//print_Link(&robot.links[0]);
	print_robot(robot);
	sciprint("\n njoints = %d", njoints);
	sciprint("\n|--------------------------------------------------------|\n");
#endif
	
	/*
	*	GET INPUTS
	*/
	// get_inputs(robot,Q,Qd,Qdd,Grav,Fext,nq);	
	
	
	
	switch (*getNbInputArgument(pvApiCtx)) {
//	switch (Rhs) {
    case 2:{ /* TAU = RNE(ROBOT, [Q QD QDD]) */ 
		/*
		 * TAU = RNE(ROBOT, [Q QD QDD])
		 */ 

		sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddrQ);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQ, &nq, &ncols, &Q);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		if (ncols != 3 * njoints)
		{
	           Scierror(999,"frne: too few cols in [Q QD QDD]");
			   return 0;
		}
		Qd = &Q[njoints*nq];
		Qdd = &Q[2*njoints*nq];

		break;
	}	
	case 3:{ /* TAU = RNE(ROBOT, [Q QD QDD], GRAV) */
		/*
		 * TAU = RNE(ROBOT, [Q QD QDD], GRAV)
		 */ 

		sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddrQ);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQ, &nq, &ncols, &Q);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		if (ncols != 3 * njoints)
		{
	           Scierror(999,"frne: too few cols in [Q QD QDD]");
			   return 0;
		}
		Qd = &Q[njoints*nq];
		Qdd = &Q[2*njoints*nq];
	/*-------------> GRAV <------------*/	
		sciErr = getVarAddressFromPosition(pvApiCtx, 3, &piAddrGrav);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrGrav, &m1, &n1, &Grav);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		if (m1*n1 != 3)
		{
			Scierror(999,"frne: gravity vector expected");
			return 0;
		}
		robot.gravity = (Vect *) malloc(sizeof(Vect));
		robot.gravity->x = Grav[0];
		robot.gravity->y = Grav[1];
		robot.gravity->z = Grav[2];
		break;
	}
    case 4: /* TAU = RNE(ROBOT, Q, QD, QDD) 
			 * TAU = RNE(ROBOT, [Q QD QDD], GRAV, FEXT)*/		 
     { 
		sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddrQ);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQ, &nq, &ncols, &Q);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}	 
        if (ncols == (3 * njoints)) {
            Qd = &Q[njoints*nq];
            Qdd = &Q[2*njoints*nq];
	/*-------------> GRAV <------------*/
		sciErr = getVarAddressFromPosition(pvApiCtx, 3, &piAddrGrav);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrGrav, &m1, &n1, &Grav);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		if (m1*n1 != 3)
		{
			Scierror(999,"frne: gravity vector expected");
			return 0;
		}
		robot.gravity = (Vect *) malloc(sizeof(Vect));
		robot.gravity->x = Grav[0];
		robot.gravity->y = Grav[1];
		robot.gravity->z = Grav[2];
	/*-------------> FEXT <------------*/
		sciErr = getVarAddressFromPosition(pvApiCtx, 4, &piAddrFext);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrFext, &m1, &n1, &Fext);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}			
        if (m1*n1 != 6)
        {
			Scierror(999,"frne: Fext vector expected");
			return 0;
		}
    } else {	/* TAU = RNE(ROBOT, Q, QD, QDD) */
		int nqd = 0, nqdd = 0; //NUMROWS(A2_IN),NUMROWS(A3_IN);
		int ncols2=0,ncols3=0;
		sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddrQ);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQ, &nq, &ncols, &Q);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}	 
		sciErr = getVarAddressFromPosition(pvApiCtx, 3, &piAddrQd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQd, &nqd, &ncols2, &Qd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
 		sciErr = getVarAddressFromPosition(pvApiCtx, 4, &piAddrQdd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQdd, &nqdd, &ncols3, &Qdd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}

		if ((nq != nqd) || (nqd != nqdd)){
			Scierror(999,"frne: Q QD QDD must be same length");
			return 0;
		}
		if ( (ncols != njoints) ||
			 (ncols2 != njoints) ||
			 (ncols3 != njoints)
		)
		{
			Scierror(999,"frne: Q must have Naxis columns");
			return 0;
		}
    }
    break;
	}
    case 5:{ /* TAU = RNE(ROBOT, Q, QD, QDD, GRAV) */
    /*
     * TAU = RNE(ROBOT, Q, QD, QDD, GRAV)
     */
        	int nqd = 0, nqdd = 0;
		int ncols2=0,ncols3=0;
		sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddrQ);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQ, &nq, &ncols, &Q);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}	 
		sciErr = getVarAddressFromPosition(pvApiCtx, 3, &piAddrQd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQd, &nqd, &ncols2, &Qd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
 		sciErr = getVarAddressFromPosition(pvApiCtx, 4, &piAddrQdd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQdd, &nqdd, &ncols3, &Qdd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		
        if ((nq != nqd) || (nqd != nqdd)){
            Scierror(999,"frne: Q QD QDD must be same length");
			return 0;
		}
        if ( (ncols != njoints) ||
             (ncols2 != njoints) ||
             (ncols3 != njoints)
        ){ 
            Scierror(999,"frne: Q must have Naxis columns");
			return 0;
		}
	/*-------------> GRAV <------------*/
		sciErr = getVarAddressFromPosition(pvApiCtx, 5, &piAddrGrav);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrGrav, &m1, &n1, &Grav);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		if (m1*n1 != 3)
		{
			Scierror(999,"frne: gravity vector expected");
			return 0;
		}
		robot.gravity = (Vect *) malloc(sizeof(Vect));
		robot.gravity->x = Grav[0];
		robot.gravity->y = Grav[1];
		robot.gravity->z = Grav[2];
        break;
    }		
    case 6:{ /* TAU = RNE(ROBOT, Q, QD, QDD, GRAV, FEXT) */
    /*
     * TAU = RNE(ROBOT, Q, QD, QDD, GRAV, FEXT)
     */
		int nqd = 0, nqdd = 0;
		int ncols2=0,ncols3=0;
		sciErr = getVarAddressFromPosition(pvApiCtx, 2, &piAddrQ);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQ, &nq, &ncols, &Q);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}	 
		sciErr = getVarAddressFromPosition(pvApiCtx, 3, &piAddrQd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQd, &nqd, &ncols2, &Qd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
 		sciErr = getVarAddressFromPosition(pvApiCtx, 4, &piAddrQdd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrQdd, &nqdd, &ncols3, &Qdd);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		
		if ((nq != nqd) || (nqd != nqdd)){
			Scierror(999,"frne: Q QD QDD must be same length");
			return 0;
	        }
		if ( (ncols != njoints) ||
	             (ncols2 != njoints) ||
	             (ncols3 != njoints) )
		{
			Scierror(999,"frne: Q must have Naxis columns");
			return 0;
		}

	/*-------------> GRAV <------------*/
		sciErr = getVarAddressFromPosition(pvApiCtx, 5, &piAddrGrav);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrGrav, &m1, &n1, &Grav);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		if (m1*n1 != 3)
		{
			Scierror(999,"frne: gravity vector expected");
			return 0;
		}
		robot.gravity = (Vect *) malloc(sizeof(Vect));
		robot.gravity->x = Grav[0];
		robot.gravity->y = Grav[1];
		robot.gravity->z = Grav[2];
	/*-------------> FEXT <------------*/
		sciErr = getVarAddressFromPosition(pvApiCtx, 6, &piAddrFext);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		sciErr = getMatrixOfDouble(pvApiCtx, piAddrFext, &m1, &n1, &Fext);
		if(sciErr.iErr)
		{
			printError(&sciErr, 0);
			return 0;
		}
		if ((m1 != 6) | (n1 != 1))
		{
			Scierror(999,"frne: Fext vector expected");
			return 0;
		}
        break;
	}
	default:{
			Scierror(999,"frne: wrong number of arguments.");
			return 0;
		}
    }
	/*FINE GET INPUTS*/
#ifdef  DEBUG	
	sciprint("\n|----------------Verifica Vettori Input------------------|");
	sciprint("\n   nq = %d",nq);
	sciprint("\n   Q =");
	print_matrix(Q,nq,njoints);
	sciprint("\n   Qd =");
	print_matrix(Qd,nq,njoints);
	sciprint("\n   Qdd =");
	print_matrix(Qdd,nq,njoints);
	sciprint("\n|--------------------------------------------------------|\n");
#endif	
	
	  
	 /* Create the matrix as return of the function */
	tau = (double*)malloc(sizeof(double) * nq * robot.njoints);
	
#define MEL(x,R,C)  (x[(R)+(C)*nq])
	/* for each point in the input trajectory */
	for (p=0; p<nq; p++) {
        /*
         * update all position dependent variables
         */
		for (j = 0; j < njoints; j++) {
 			l = &robot.links[j];
			switch (l->sigma) {
				case REVOLUTE:
					rot_mat(l, MEL(Q,p,j)+l->offset, l->D, robot.dhtype);
					break;
				case PRISMATIC:
					rot_mat(l, l->theta, MEL(Q,p,j)+l->offset, robot.dhtype);
					break;
			}
#ifdef  DEBUG
			print_matrix(&l->R, 3, 3);
			print_vect(&l->r, 3);
#endif
		}
		newton_euler(&robot, &tau[p], &Qd[p], &Qdd[p], Fext, nq);
	}

    
#ifdef  DEBUG
 	sciprint("\n  FRNE:\t");
	print_matrix(tau,nq,njoints);
#endif

	sciErr = createMatrixOfDouble(pvApiCtx, nbInputArgument(pvApiCtx) + 1, nq, njoints, tau);

	AssignOutputVariable(pvApiCtx, 1) = nbInputArgument(pvApiCtx) + 1;
	free_robot(robot);
	free(tau);
	return 0;
}


void print_vect(double *vect,int length){
		int i;
		sciprint("\n Vect = ");
		for (i=0;i<length;++i)
		{
			sciprint(" %.2f ",vect[i]); 
		}
		sciprint("\n");
}

void print_matrix(double *matrix,int m,int n)
{
	int i,j;
	sciprint("\n Vect = ");
	sciprint("\t\t(size: %d, %d):\n",m, n);
	for (j=0; j<m ; ++j)
	{
		for (i=0;i<n;++i)
		{
			sciprint(" %.2f ",matrix[i*m + j]); 
		}
		sciprint("\n");
	}
}

/*
 *  Written by;
 *
 *      Peter I. Corke
 *      CSIRO Division of Manufacturing Technology
 *      Preston, Melbourne.  Australia. 3072.
 *
 *      pic@mlb.dmt.csiro.au
 *
 *  Permission to use and distribute is granted, provided that this message
 * is retained, and due credit given when the results are incorporated in
 * publised work.
 *
 */

/**
 * Return the link rotation matrix and translation vector.
 *
 * @param l Link object for which R and p* are required.
 * @param th Joint angle, overrides value in link object
 * @param d Link extension, overrides value in link object
 * @param type Kinematic convention.
 */
void
rot_mat (
    Link    *l,
    double  th,
    double  d,
    DHType  type
) {
    double      st, ct, sa, ca;

#ifdef  sun
    sincos(th, &st, &ct);
    sincos(l->alpha, &sa, &ca);
#else
    st = sin(th);
    ct = cos(th);
    sa = sin(l->alpha);
    ca = cos(l->alpha);
#endif

    switch (type) {
case STANDARD:
    l->R.n.x = ct;      l->R.o.x = -ca*st;  l->R.a.x = sa*st;
    l->R.n.y = st;      l->R.o.y = ca*ct;   l->R.a.y = -sa*ct;
    l->R.n.z = 0.0;     l->R.o.z = sa;      l->R.a.z = ca;

    l->r.x = l->A;
    l->r.y = d * sa;
    l->r.z = d * ca;
    break;
case MODIFIED:
    l->R.n.x = ct;      l->R.o.x = -st;     l->R.a.x = 0.0;
    l->R.n.y = st*ca;   l->R.o.y = ca*ct;   l->R.a.y = -sa;
    l->R.n.z = st*sa;   l->R.o.z = ct*sa;   l->R.a.z = ca;

    l->r.x = l->A;
    l->r.y = -d * sa;
    l->r.z = d * ca;
    break;
    }
}
