/*  Copyright (C) 2020 - Corrado Guarino Lo Bianco
*
*  This file is part of the Robotics Toolbox for Scilab.
*
*  Robotics Toolbox is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Robotics Toolbox is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
*  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
*  by Peter Corke.
*/

#include <memory.h>
#include "scicos_block4.h"
#include "findbox.h"

void block_jacobn(scicos_block *block,int flag)
{
  /* Copyright INRIA
     
  Scicos block simulator
  discrete state space linear system simulator
  rpar(1:nx*nx)=A
  rpar(nx*nx+1:nx*nx+nx*nu)=B
  rpar(nx*nx+nx*nu+1:nx*nx+nx*nu+nx*ny)=C */
  
/*  int un=1,lb,lc,ld;
  int nz=GetNdstate(block);
  double* z=GetDstate(block);
  double* rpar=GetRparPtrs(block);
  double* y=GetRealOutPortPtrs(block,1);
  double* u=GetRealInPortPtrs(block,1);
  int noutsz=GetOutPortRows(block,1);
  int ninsz=GetInPortRows(block,1);
;
  
  
  lb=nz*nz; */
  double* rpar=GetRparPtrs(block);
  double* y1=GetRealOutPortPtrs(block,1);
  double* y2=GetRealOutPortPtrs(block,2);
  double* y3=GetRealOutPortPtrs(block,3);
  double* y4=GetRealOutPortPtrs(block,4);
  double* z=GetDstate(block);
  double* ref=GetRealInPortPtrs(block,1);
  double* bounds=GetRealInPortPtrs(block,2);
  robot* w = NULL;

  if (flag == OutputUpdate || flag == ReInitialization){
	*y1 = z[0];
	*y2 = z[1];
	*y3 = z[2];
	*y4 = z[3];
  } 
  else if (flag == StateUpdate){

	double *x2dR, *xdR, *xR, u;
	double *Up, *Um, *x2dP, *x2dM, *xdP, *xdM;
	double yy[3], zz[3];
	double z2OP, z2OM, z2HP, z2HM;
	double z3OP, z3OM, z3HP, z3HM, z3P, z3M;
	double d1, gamma1, kappa1, m1, sigma1;
	double d2, gamma2, kappa2, m2, sigma2;
	double alpha, beta;
	double sigma_overline, sigma;
	double *Tsamp, *Tsamp2, *Tsamp3;
	double *invTsamp, *invTsamp2, *invTsamp3;

	int h, k, j, eta, xi;
	double rho, num1, num2, num3, num4, den;
	double xtmp, xdtmp, x2dtmp;
	
	/* load reference pointers */

	xR   = ref;
	xdR  = ref+1;
	x2dR = ref+2;
	
	/* load bounds pointers */

	xdP  = bounds;
	xdM  = bounds+1;
	x2dP = bounds+2;
	x2dM = bounds+3;
	Up   = bounds+4;
	Um   = bounds+5;

	/* load useful constants evaluated during initialization */

	w = (double*) *block->work;	
	Tsamp = w;
	Tsamp2 = w+1;
	Tsamp3 = w+2;
	invTsamp = w+3;
	invTsamp2 = w+4;
	invTsamp3 = w+5;


	/* Integrity checks */

	/* If the velocity of the reference signal is unfeasible
	   exchange its poiter with the one of the proper bound pointer */

	if (*xdP < *xdR)
	    xdR = xdP;
	if (*xdM > *xdR)
	    xdR = xdM;

	/* If the acceleration of the reference signal is unfeasible
	   exchange its poiter with the one of the proper bound pointer */

	if (*x2dP < *x2dR)
	    x2dR = x2dP;
	if (*x2dM > *x2dR)
	    x2dR = x2dM;
	
	/* evaluate the command signal for the filter: transformations phase */

	yy[0] = z[0] - *xR;
	yy[1] = z[1] - *xdR;
	yy[2] = z[2] - *x2dR;
	    
	zz[0] = *invTsamp3 * yy[0] + (*invTsamp2) * yy[1] + (*invTsamp/3.0) * yy[2];
	zz[1] = *invTsamp2 * yy[1] + (*invTsamp/2.0) * yy[2];
	zz[2] = *invTsamp * yy[2];

	z3M = (*x2dM - *x2dR) * (*invTsamp);
	z3P = (*x2dP - *x2dR) * (*invTsamp);

	/* overline */
	z2OP = (*xdP - *xdR) * (*invTsamp2) - *x2dR * (*invTsamp) / 2.0;
	z2OM = (*xdM - *xdR) * (*invTsamp2) - *x2dR * (*invTsamp) / 2.0;

	/* overline */
	z3OP = -(*x2dR) * (*invTsamp);
	z3OM = -(*x2dR) * (*invTsamp);

	/* hat */
	z3HP = z3P - z3OP;
	z3HM = z3M - z3OM;

	/* hat */
	z2HP = -ceil(-z3HP / *Um) * (z3HP + *Um / 2.0 * (ceil(z3HP / *Um) - 1.0));
	z2HM = -ceil(-z3HM / *Up) * (z3HM + *Up / 2.0 * (ceil(z3HM / *Up) - 1.0));

	/* Compute sigma1 */
	d1 = zz[1] - z2OP;

	if (d1 < z2HP)
	    gamma1 = z2HP;
	else if (d1 > z2HM)
	    gamma1 = z2HM;
	else
	    gamma1 = d1;

	if (gamma1 <= 0.0)
	    kappa1 = *Um;
	else
	    kappa1 = *Up;

	m1 = floor(1.0 / 2.0 + sqrt(1.0 / 4.0 + 2.0 * fabs(gamma1 / kappa1)));

	sigma1 = -gamma1 / m1 - (m1 - 1.0) * kappa1 / 2.0 + z3OP;

	/* Compute sigma2 */
	d2 = zz[1] - z2OM;

	if (d2 < z2HP)
	    gamma2 = z2HP;
	else if (d2 > z2HM)
	    gamma2 = z2HM;
	else
	    gamma2 = d2;

	if (gamma2 <= 0.0)
	    kappa2 = *Um;
	else
	    kappa2 = *Up;

	m2 = floor(1.0 / 2.0 + sqrt(1.0 / 4.0 + 2.0 * fabs(gamma2 / kappa2)));

	sigma2 = -gamma2 / m2 - (m2 - 1.0) * kappa2 / 2.0 + z3OP; 
	struct findRet a, *ret;
	ret = &a;
	findbox(ret, zz[0], zz[1], *Um, *Up, z3M, z3P);
	h       = ret->h;
	k       = ret->k;
	j       = ret->s;
	rho  = ret->alpha;
	eta     = ret->eta;
	xi      = ret->mu;

	if(eta > 0) {
	    alpha = *Um;
	    beta  = *Up;
	} else {
	    alpha = *Up;
	    beta  = *Um;
	}
	/* Control law sigma3 and sigma4, determing sigma_overline */
	if (xi == -1) /* sigma3 */
	    sigma_overline = - 2.0/((k+h)*h)*zz[0] - (2.0*h+k-1)/((k+h)*h)*zz[1] - k*(1.0-pow(k, 2.0))*alpha/(6.0*h*(k+h)) - 
			      (2.0*pow(h, 3.0)-3*pow(h, 2.0)+h+3*pow(h, 2.0)*k-3.0*k*h)*beta/(6.0*h*(k+h));  
	else { /* sigma4 */
	    num1 = (6.0*pow(j, 2.0)+(-6.0+12.0*k)*j + 6.0*k*(k-1.0))*pow(rho,2.0)+(6.0*pow(j,2.0)+(6.0*k-12.0)*j)*pow(k,2.0)+
			  (12.0*pow(j,2.0)*k+(18.0*(-k+pow(k,2.0)))*j+4.0*pow(k,3.0)+2.0*k-18.0*pow(k,2.0))*rho-6.0*pow(k,3.0)+
			   5.0*pow(k,2.0)+pow(k,4.0);
	    num2 = 6.0*h*(-h+1.0)*(j*rho+rho*k+j*k)-rho*h*(4.0-6.0*h+2.0*pow(h,2.0))-h*k*(7.0-9.0*h+2*pow(h,2.0))+
			  3.0*pow(k,2.0)*h*(-h+1.0);
	    num3 = -12.0*(k+rho);
	    num4 = (12.0*(h+j))*(-k-rho)+12.0*rho*(-k+1.0)+6.0*k*(3.0-k);
	    den  = h*(k*(k-2.0+h+2.0*j)+rho*(2.0*j-1+h+2.0*k));

	    sigma_overline = (num1*alpha + num2*beta + num3*zz[0] + num4*zz[1])/(6.0*den);
	}

	/* sigma5 */
	if (sigma_overline > z3P)
	    sigma_overline = z3P;
	/* sigma6 */
	if (sigma_overline < z3M)
	    sigma_overline = z3M;

	/* Defining the control law: sigma */
	if (sigma1 < sigma_overline)
	    sigma = sigma1;
	else if (sigma_overline < sigma2)
	    sigma = sigma2;
	else
	    sigma = sigma_overline;

	if ((zz[2] - sigma) >= 0)
	    u = -(*Um) * sat((zz[2] - sigma)/ (*Um));
	else
	    u = -(*Up) * sat((zz[2] - sigma)/ (*Up));

	/* Status update */
	xtmp   = z[0];
	xdtmp  = z[1];
	x2dtmp = z[2];

	z[0] = xtmp + *Tsamp * xdtmp + *Tsamp2 * x2dtmp / 2.0 + *Tsamp3 * u / 6.0;
	z[1] = xdtmp + *Tsamp * x2dtmp + *Tsamp2 * u / 2.0;
	z[2] = x2dtmp + *Tsamp * u;
	z[3] = u;
  }
  else if (flag == Initialization){/* the workspace for temp storage
		      */
    robot* w = NULL;

    *block->work = scicos_malloc(sizeof(robot));

    if (*block->work == NULL){
        set_block_error(-16);
        return;
    }

    w = (robot*) *block->work;

    robot_init(w , block)


    /*sciprint("xdP =%.3f\txdM = %.3f\tx2dP = %.3f\tx2dM = %.3f\tUp = %.3f\tUm = %.3f \n",w[0],w[1],w[2],w[3],w[4],w[5]);*/

  }
  else if (flag == Ending){

    robot* w = NULL;

    w = (robot*) *block->work;

    scicos_free(w->links);
    scicos_free(*block->work);
  }
}



