/*  Copyright (C) 2020 - Corrado Guarino Lo Bianco
*
*  This file is part of the Robotics Toolbox for Scilab.
*
*  Robotics Toolbox is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Robotics Toolbox is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
*  This toolbox is inspired to the Robotics Toolbox v9 for MATLAB 
*  by Peter Corke.
*/

#include <math.h> /* atan2 */
#include <float.h> /* DBL_EPSILON */

int t2zyz(double *t, double *phi, double *theta, double *psi)
{

	double sp,cp;
	
    //	 Method as per Paul, p 69.
    //	 phi = atan2(ay, ax)
    //	 Only positive phi is returned.
    if (fabs(t[8]) < DBL_EPSILON & fabs(t[9]) < DBL_EPSILON)
    {
        // singularity
        *phi = 0.0;
        *theta = atan2(0.0, t[10]);
        *psi = atan2(t[1], t[5]);
    }
    else
    {
        *phi = atan2(t[9], t[8]);
        sp = sin(*phi);
        cp = cos(*phi);
        *theta = atan2(cp*t[8] + sp*t[9], t[10]);
        *psi = atan2(-sp*t[0] + cp*t[1], -sp*t[4] + cp*t[5]);
    } 
      
    return 0;
}

